/*
 * Copyright (C) 2005-2009 Makoto Sato. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.objectfanatics.assertion.weaver.api.core;

import jp.objectfanatics.assertion.weaver.api.exception.IllegalParameterConstraintAnnotationOverridingException;
import jp.objectfanatics.assertion.weaver.api.exception.IllegalReturnValueConstraintAnnotationOverridingException;
import jp.objectfanatics.assertion.weaver.api.exception.IllegalUseOfParameterConstraintAnnotationException;
import jp.objectfanatics.assertion.weaver.api.exception.IllegalUseOfReturnValueConstraintAnnotationException;
import jp.objectfanatics.assertion.weaver.api.exception.NoWeavingNeededException;
import jp.objectfanatics.assertion.weaver.api.exception.NotClassFileException;

// Checked 09.26.2009. Delete this line before release.
/**
 * Weaver for weaving assertion functions into a class file bytes.
 * 
 * @author Makoto Sato
 */
public interface Weaver {
	
	/**
	 * Weaves assertion functions into a class file bytes if needed.
	 * 
	 * @param classFileBytes
	 *            the class file bytes to be woven
	 * @return the woven class file bytes
	 * @throws NoWeavingNeededException
	 * @throws NotClassFileException
	 * @throws IllegalUseOfReturnValueConstraintAnnotationException
	 * @throws IllegalUseOfParameterConstraintAnnotationException
	 * @throws IllegalReturnValueConstraintAnnotationOverridingException
	 * @throws IllegalParameterConstraintAnnotationOverridingException
	 * @throws IllegalArgumentException
	 *             if <code>classFileBytes</code> is null
	 */
	byte[] weave(byte[] classFileBytes) throws NoWeavingNeededException, NotClassFileException, IllegalUseOfReturnValueConstraintAnnotationException, IllegalUseOfParameterConstraintAnnotationException, IllegalReturnValueConstraintAnnotationOverridingException, IllegalParameterConstraintAnnotationOverridingException;
}
